-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp, SQLskills.com
-- All Rights Reserved.
-- 
-- For more scripts, sample code and additional training resources,
-- go to http://www.SQLskills.com
--	Immerse yourself in SQL Server
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

-- USE TestDB -- Use a Testing Database
-- go

IF OBJECTPROPERTY(object_id('Employee'), 'IsUserTable') = 1
	DROP TABLE Employee
go

CREATE TABLE Employee
(
	EmpID	int	identity(10000,3)
		             CONSTRAINT EmployeePK
				       PRIMARY KEY --CLUSTERED (EmpID) 
			      CONSTRAINT EmployeeEmpIDCk
			     		CHECK (EmpID BETWEEN 10000 AND 1000000)
			      CONSTRAINT EmployeeEmpIDCk2
			     		CHECK ((EmpID - 10000) % 3 = 0)
-- 			      CONSTRAINT EmployeeEmpIDCk
-- 			     		CHECK (EmpID BETWEEN 10000 AND 1000000 
-- 								AND (EmpID - 10000) % 3 = 0)
,
	LName	varchar(20)	NOT NULL,
	FName	varchar(20)	NOT NULL,	
-- 	DeptID	int		NOT NULL
-- 				CONSTRAINT EmployeeDeptIDtoDeptFK
-- 					REFERENCES dbo.Department(DeptID), 
	SSN	char(11)	NULL
				CONSTRAINT EmployeeSSNUK
					UNIQUE -- NONCLUSTERED (SSN)
				CONSTRAINT EmployeeSSNCK
					CHECK (SSN LIKE '[0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9][0-9][0-9]' ),
	MgrID	int		NOT NULL
				CONSTRAINT EmployeeMgrIDtoEmpIDFK
					--FOREIGN KEY (MgrID)
					REFERENCES dbo.Employee(EmpID),
					--ON UPDATE CASCADE, -- NOT allowed for self referencing keys
	Phone	char(14)	NOT NULL
				CONSTRAINT EmployeePhoneCK
					CHECK (Phone LIKE '([2-9a-z][0-9a-z][0-9]) [2-9][0-9][0-9]-[0-9][0-9][0-9][0-9]' 
						OR Phone LIKE '(___) ___-____' 
						OR Phone LIKE '([2-9a-z]__) [a-z2-9]__-____')
				CONSTRAINT EmployeePhoneDefault
					DEFAULT '(425) 882-8080',
	Salary	money		NOT NULL
				CONSTRAINT EmployeeSalaryMaxCk
					CHECK (Salary <= 500000),
	Bonus	money		NOT NULL,
	CONSTRAINT EmployeeBonusSalaryCk
		CHECK (Bonus <= Salary *.15),
	State	char(2)		NOT NULL
				CONSTRAINT EmployeeStateCk
					CHECK (State IN ('WA', 'OR', 'CA'))
				CONSTRAINT EmployeeStateDflt
					DEFAULT 'WA',
	HireDate	datetime	NOT NULL
					CONSTRAINT EmployeeHireDateDflt
						DEFAULT getdate(),
						-- DEFAULT CURRENT_TIMESTAMP 
						-- see niladic functions in BOL
	Payterms	tinyint		NULL,
	CONSTRAINT EmployeeHireDateAndPayTermsCk
		CHECK((HireDate >= '1/1/2000' AND PayTerms <= 30) OR 
			(HireDate < '1/1/2000' AND PayTerms <= 45)),
	InsertedBy	varchar(60)	NOT NULL
					CONSTRAINT EmployeeInsertedByDflt
						DEFAULT host_name() + '\' + suser_sname() + '\' + user_name() 
-- IF TABLE LEVEL CONSTRAINT OR via ALTER...
-- 	, 	CONSTRAINT EmployeeInsertedByDflt
-- 			DEFAULT host_name() + '\' + suser_sname() + '\' + user_name() FOR InsertedBy
)
go
sp_helpconstraint employee
go
ALTER TABLE Employee
ADD 
      CONSTRAINT EmployeeEmpIDCk2
     		CHECK ((EmpID - 10000) % 3 = 0)
go
ALTER TABLE Employee
ADD 
	CONSTRAINT EmployeeStateCk2
		CHECK (State IN ('IL', 'WI', 'MI'))
go
ALTER TABLE Employee
ADD 
CONSTRAINT EmployeeEmpID2Ck
			     		CHECK ((EmpID-10000) % 3 = 0)
go
INSERT Employee (Lname, Fname, bonus, state, ssn, mgrid, salary, payterms)
	VALUES ('tripp', 'linda', 100, 'IL', '123-45-6789', 10000, 1000, 10)
-- Violates EmployeeStateCK
go
INSERT Employee (Lname, Fname, bonus, state, ssn, mgrid, salary, payterms)
	VALUES ('tripp', 'linda', 100, 'WA', '123-45-6789', 10000, 1000, 10)
-- Violates EmployeeStateCK2
go
ALTER TABLE Employee
DROP CONSTRAINT EmployeeStateCk2
go
INSERT Employee (Lname, Fname, bonus, state, ssn, mgrid, salary, payterms)
	VALUES ('tripp', 'linda', 100, 'WA', '123-45-6789', 10000, 1000, 10)
-- Now this doesn't violate EmployeeStateCK (but there are still other problems)
go
ALTER TABLE Employee
	NOCHECK CONSTRAINT EmployeeMgrIDtoEmpIDFK
go
DBCC CHECKIDENT('Employee', 'NORESEED') -- use this for max current mgrid
go
INSERT Employee (Lname, Fname, bonus, state, ssn, mgrid, salary, payterms)
	VALUES ('tripp', 'linda', 100, 'WA', '123-45-6789', 10009, 1000, 10)
go
select * from Employee
go
ALTER TABLE Employee
	CHECK CONSTRAINT EmployeeMgrIDtoEmpIDFK
go
DBCC CHECKCONSTRAINTS ('EmployeeMgrIDtoEmpIDFK')
DBCC CHECKCONSTRAINTS ('Employee') -- to check all constraints
	WITH ALL_CONSTRAINTS
go
SELECT * FROM employee WHERE MgrID = '10009' 
go
UPDATE Employee
	SET MgrID = 10012
	WHERE EmpID = 10012
go
ALTER TABLE Employee
ADD
	CONSTRAINT SalaryDF
		DEFAULT 600000 FOR Salary	
go
INSERT employee (Lname, Fname, bonus, state, ssn, mgrid, salary, payterms)
	VALUES ('tripp', 'kim', 100, 'WA', '123-45-6790', 10012, DEFAULT, 10)
go

SELECT * FROM Employee
go


UPDATE employee
	SET ssn = '987-65-4321'
	WHERE empid = 10018
go

SELECT * FROM employee 
	WHERE mgrid NOT IN (SELECT empid FROM employee)
go

UPDATE employee
	SET mgrid = 10010
	WHERE empid = 10018
go

ALTER TABLE employee
DROP CONSTRAINT EmployeeMgrIDtoEmpIDFK

ALTER TABLE employee
--with nocheck
ADD CONSTRAINT EmployeeMgrIDtoEmpIDFK
	FOREIGN KEY (MgrID)
	REFERENCES dbo.Employee(EmpID)

-- misc stuff

10003	= 3
10006	= 6
10009	= 9...

select host_name() + '\' + suser_sname() + '\' + user_name()
select host_name() + '\' + suser_name() + '\' + user_name()
select host_name() + '\' + ISNULL(suser_name(), 'NTAuth')  + '\' + user_name()
dbcc useroptions
sp_helpconstraint authors
